from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import \
    longest_strike_above_mean
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class LongestStrikeAboveMean(AggregationPrimitive):
    """Returns the length of the longest consecutive subsequence in x that is
    bigger than the mean of x.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.longest_strike_above_mean
    """
    name = "longest_strike_above_mean"
    input_types = [ColumnSchema(semantic_tags={'numeric'})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={'numeric'})
    stack_on_self = False

    def get_function(self):
        return longest_strike_above_mean



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import sum_values
from woodwork.column_schema import ColumnSchema


class SumValues(AggregationPrimitive):
    """Calculates the sum over the time series values.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.sum_values
    """
    name = "sum_values"
    input_types = [ColumnSchema(semantic_tags={'numeric'})]
    return_type = ColumnSchema(semantic_tags={'numeric'})
    stack_on_self = False

    def get_function(self):
        return sum_values



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import count_above_mean
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class CountAboveMean(AggregationPrimitive):
    """Returns the number of values in x that are higher than the mean of x

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.count_above_mean
    """
    name = "count_above_mean"
    input_types = [ColumnSchema(semantic_tags={'numeric'})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={'numeric'})
    stack_on_self = False

    def get_function(self):
        return count_above_mean



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import \
    first_location_of_minimum
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class FirstLocationOfMinimum(AggregationPrimitive):
    """Returns the first location of the minimal value of x. The position is
    calculated relatively to the length of x.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.first_location_of_minimum
    """
    name = "first_location_of_minimum"
    input_types = [ColumnSchema(semantic_tags={'numeric'})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={'numeric'})
    stack_on_self = False

    def get_function(self):
        def function(x):
            return first_location_of_minimum(x.to_numpy())

        return function
